#include "head.h"

void Usage(void)
{
  printf("head %s created by Jorgen Bosman\n",HEAD_VERSION);
  printf("Usage: head [options] file\n");
  printf("Options:\n");
  printf("  -n x  : Displays the last x lines of the file\n");
  exit(-1);
}

void DisplayLastError(void)
{
  LPVOID lpMsgBuf;
  
  FormatMessage( 
    FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
    NULL,
    GetLastError(),
    MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
    (LPTSTR) &lpMsgBuf,    0,    NULL );
  printf("%s\n",lpMsgBuf);
  LocalFree( lpMsgBuf );
}

int Head(char *filename,Options options)
{
  HANDLE f;
  unsigned lines=0,x,oldpos,lines_skipped=0;
  char buf[512];
  DWORD bytes_read=0;

  f=CreateFile(filename,
               GENERIC_READ,
               FILE_SHARE_DELETE|FILE_SHARE_READ|FILE_SHARE_WRITE,
               NULL,
               OPEN_EXISTING,
               FILE_ATTRIBUTE_NORMAL,
               NULL);
  if (f==INVALID_HANDLE_VALUE)
  {
    DisplayLastError();
    return -1;
  }
  while (lines<options.nfirstlines)
  {
    if (!ReadFile(f,buf,511,&bytes_read,NULL))
    {
      DisplayLastError();
      CloseHandle(f);
      return -1;
    }    
    if (bytes_read==0)
      break;
    buf[bytes_read]='\0';
    oldpos=0;
    for (x=0;x<bytes_read;x++)
    {
      if (buf[x]=='\n')
      {
        buf[x]='\0';
        if (!options.nskiplines)
        {
          printf("%s\n",&buf[oldpos]);
          fflush(stdout);
          lines++;
        }
        else
        {
          lines_skipped++;
          if (lines_skipped>=options.nskiplines)
            options.nskiplines=0;
        }
        oldpos=x+1;
        if (lines>=options.nfirstlines)
          break;
      }
    }
    if (!options.nskiplines && lines<options.nfirstlines)
    {
      printf("%s",&buf[oldpos]);
      fflush(stdout);
    }
  }
  CloseHandle(f);
  return 0;
}

int main(int argc,char *argv[])
{
  int x;
  Options options={10,0};
  char *filename=NULL;

  for (x=1;x<argc;x++)
  {
    if (argv[x][0]=='-')
    {
      if (!strcmp(argv[x],"-n"))
      {
        x++;
        if (x<argc)
          options.nfirstlines=atoi(argv[x]);
        else
          Usage();
      }
      else if (!strcmp(argv[x],"-s"))
      {
        x++;
        if (x<argc)
          options.nskiplines=atoi(argv[x]);
        else
          Usage();
      }
    }
    else if (filename==NULL)
      filename=strdup(argv[x]);
    else
      Usage();
  }
  if (filename==NULL)
    Usage();

  x=Head(filename,options);

  free(filename);
  return x;
}